/*


   TIGA test program
   Demonstrates virtual screen handling

   This program allocates memory for a virtual screen whose dimensions
   are bigger than the visible screen. Then it creates a simple test
   picture in the virtual screen. After this you can move with the mouse
   across the virtual screen. This is done by copying the selected area
   from virtual screen to visible screen. A 34020 processor can do this
   very fast, for example a 1280 x 1024 16 colors screen in 49 ms, so
   the copy operations are nearly invisible and the movements with mouse
   are looking very smooth.
   If there is no mouse driver installed or not enough memory found for a
   big virtual screen, the program aborts.

   Compile in small model and link with AI.LIB

   ELSA GmbH Aachen
   SK 20-12-90

*/

#include <conio.h>
#include <stdio.h>
#include <math.h>
#include <dos.h>

#include <tiga.h>
#include <extend.h>
#include <typedefs.h>

typedef struct
{
   short x,y;           /* coordinates */
   short left, right;   /* buttons     */
   short x1,y1, x2,y2;  /* boundary    */
} MOUSE;

MOUSE mouse;
union REGS regs;

/* this function checks if a mouse driver is installed */
int check_mouse( void )
{
    regs.x.ax = 0;
    int86( 0x33, &regs, &regs);
    return( regs.x.ax );
}

/* get mouse coordinates and buttons */
void get_mouse( MOUSE * mouse )
{
    /* get mouse coordinates */
    regs.x.ax = 11;
    int86( 0x33, &regs, &regs);
    mouse->x += regs.x.cx;
    mouse->y += regs.x.dx;

    /* ensure the mouse stays within the screen boundary */
    if ( mouse->x < mouse->x1 )
      mouse->x = mouse->x1;
    if ( mouse->x > mouse->x2 )
      mouse->x = mouse->x2;
    if ( mouse->y < mouse->y1 )
      mouse->y = mouse->y1;
    if ( mouse->y > mouse->y2 )
      mouse->y = mouse->y2;

    /* get the mouse buttons */
    regs.x.ax = 3;
    int86( 0x33, &regs, &regs);
    mouse->left  = regs.h.bl & 1;
    mouse->right = ( regs.h.bl & 2 ) >> 1;
}


void main( int argc, char **argv )
{
  int    c, i, v, x, y;
  ulong  n;
  char   buf[80];
  PTR    newbase, zoombase;
  CONFIG config;
  ushort w, h, pitch, psize, vx, vy, oldmode;

  printf( "\nDemonstration of offscreen and heap drawing capabilities"
	  " with ELSA TIGA 2.01-E\n" );

  if( !check_mouse() )
  { printf("\nNo mouse driver installed, abort ..." );
    exit( 1 );
  }

  printf( "\nInitialize TIGA ... " );
  if( ( v = tiga_set( CD_OPEN ) ) < 0 )
  { printf( "TIGA CD error : %d\n", v );
    exit( 1 );
  }
  if( get_videomode() != TIGA )
  { oldmode = get_videomode();
    if( ( v = set_videomode( TIGA, INIT | CLR_SCREEN ) ) < 0 )
    { printf( "TIGA GM error : %d\n", v );
      tiga_set( CD_CLOSE );
      exit( 1 );
    }
  }

  get_config( &config );
  psize = config.mode.disp_psize;
  w     = config.mode.disp_hres;
  h     = config.mode.disp_vres;


  /* Get maximum size of allocatable memory */
  n = gsp_maxheap();

  /* Calculate virtual screen width */
  vx = 1 << lmo( (ushort)sqrt( (double)n * 8 / psize ) ) + 1;

  /* Define pitch of virtual screen */
  pitch = vx * psize;

  /* Determine height of virtual screen, width = vx */
  vy = (ushort)( n * 8L / (ulong)pitch );
  if( vy <= h )
  { printf( "\nNot enough extended memory on board, abort ... ");
    set_videomode( oldmode, INIT );
    tiga_set( CD_CLOSE );
    exit( 1 );
  }

  printf( "\nDimension of virtual screen is %d x %d pixels.", vx, vy );

  /* Setup mouse boundaries */
  mouse.x1 = mouse.y1 = 0;
  mouse.x2 = vx - w;
  mouse.y2 = vy - h;

  clear_screen( 0 );
  x = 0;
  y = 0;

  /* Allocate memory for virtual screen */
  n = (ulong)pitch * (ulong)vy / 8L;
  newbase = gsp_malloc( n );

  /* Load extended primitives */
  printf( "\nLoad TIGA extended primitives ... " );
  if( ( v = install_primitives() ) < 0 )
  { printf( "Graphics library load error : %d\n", v );
    set_videomode( oldmode, INIT );
    tiga_set( CD_CLOSE );
    exit( 1 );
  }

  /*
  ** Set destination bitmap to virtual screen and
  ** source bitmap to visible screen.
  */
  set_dstbm( newbase, pitch, vx, vy, psize );
  set_srcbm( 0, 0, 0, 0, 0 );
  set_clip_rect( vx, vy, 0, 0 );

  /* Draw into virtual screen if no command line paramters exists */
  if( argc == 1 )
  { printf("\nDraw into virtual screen, please wait ... ");

    /* Clear virtual screen */
    set_fcolor( 0 );
    fill_rect( vx, vy, 0, 0 );

    /* Draw some lines into virtual screen */
    c = 1;
    for( x = 0; x < vx; x += 10 )
    { set_fcolor( c );
      draw_line( x, 0, vx - x, vy - 1 );
      if( ++c == 16 )
	c = 1;
    }
    for( y = 0; y < vy - 1; y += 10 )
    { set_fcolor( c );
      draw_line( 0, y, vx - 1, vy - y );
      if( ++c == 16 )
	c = 1;
    }

    /* Draw screen coordinates into virtual screen */
    set_colors( 14, 1 );
    for( y = 0; y < vy; y+=200 )
      for( x = 0; x < vx; x+=200 )
      { sprintf( buf, "%d,%d", x, y );
	text_out( x, y, buf );
      }
  }

  /* And now copy parts of virtual screen to visible screen */
  printf("\nMove with mouse around the virtual screen, press any key to abort ... ");
  swap_bm();
  while( !kbhit() )
  { get_mouse( &mouse );
    bitblt( w, h, mouse.x, mouse.y, 0, 0 );
  }
  getch();

  printf( "\nDo you want to see the virtual memory"
	  " in a compressed format ? (y/n) " );
  putch( c = getch() );
  if( toupper( c ) == 'Y' )
  { zoombase = gsp_malloc( pitch / 8 );
    /* Increase timeout value, because the compress operation takes much time */
    set_timeout( 20000 );
    clear_screen( 0 );
    set_ppop( 20 );
    zoom_rect( vx, vy, 0, 0, w, h, 0, 0, zoombase );
    set_ppop( 0 );
    gsp_free( zoombase );
    set_timeout( 5000 );
  }

  /* Free allocated memory */
  gsp_free( newbase );

  printf("\nPress any key to return to DOS ... " );
  getch();
  set_videomode( oldmode, INIT );
  tiga_set( CD_CLOSE );
}

